/*!
   memfifo -- implementation of EZ-USB slave FIFO's (input and output) a FIFO using the BRAM of ZTEX USB-FPGA Modules 2.01
   Copyright (C) 2009-2014 ZTEX GmbH.
   http://www.ztex.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!*/
#include[ztex-conf.h]	// Loads the configuration macros, see ztex-conf.h for the available macros
#include[ztex-utils.h]	// include basic functions

// configure endpoint 2, in, quad buffered, 512 bytes, interface 0
EP_CONFIG(2,1,BULK,IN,512,4);

// configure endpoint 6, out, double buffered, 512 bytes, interface 0
EP_CONFIG(6,2,BULK,OUT,512,4);

// select ZTEX USB FPGA Module 2.01 as target  (required for FPGA configuration)
IDENTITY_UFM_2_01(10.18.0.0,0);	 

// this product string is also used for identification by the host software
#define[PRODUCT_STRING]["passthru for UFM 2.01"]

DISABLE_MAC_EEPROM;
// enable Flash support
//ENABLE_FLASH;

#define[MT_RESET][IOA7]
//#define[MT_MODE0][IOA0]
//#define[MT_MODE1][IOA1]

__xdata BYTE run;
#define[PRE_FPGA_RESET][PRE_FPGA_RESET
    run = 0;
]


// this is called automatically after FPGA configuration
#define[POST_FPGA_CONFIG][POST_FPGA_CONFIG
    reset ();
]


//   Usage:
//     ADD_EP0_VENDOR_COMMAND((<request number>,,<code executed after setup package received>,,<code executed after data package received>''));
//   Example:
//     ADD_EP0_VENDOR_COMMAND((0x33,,initHSFPGAConfiguration();,,));;

//// set mode
//ADD_EP0_VENDOR_COMMAND((0x80,,
//	IOA = SETUPDAT[2] & 3;
//,,
//	NOP;
//));;



__xdata BYTE spi_addr;
__xdata BYTE spi_write_bytes;

#define[SPI_MISO][bmBIT0]
#define[SPI_SCK][bmBIT1]
#define[SPI_CS][bmBIT2]
#define[SPI_IOUPDATE][bmBIT3]
#define[SPI_MOSI][bmBIT4]
#define[SPI_SD_T][bmBIT5]
#define[SPI_PORT][IOC]
#define[SPI_OE][OEC]


void spi_init() {
	SPI_OE   = SPI_CS|SPI_SCK|SPI_IOUPDATE|SPI_MOSI|SPI_SD_T;
	SPI_PORT = SPI_CS|SPI_SCK|SPI_MOSI;
}


void spi_write_byte(unsigned char byte)
{
    unsigned char mask = 0x80;
    do {
        SPI_PORT &= ~SPI_SCK; // SCK: \_
        SYNCDELAY;

        if(byte & mask)
            SPI_PORT |= SPI_MOSI;
        else
            SPI_PORT &= ~SPI_MOSI;
        SYNCDELAY;

        SPI_PORT |= SPI_SCK; // SCK: _/
        SYNCDELAY;
        mask = mask >> 1;
    } while(mask != 0);
}

void spi_read_byte(xdata BYTE *buf)
{
    unsigned char mask = 0x80;
    *buf = 0x00;
    do {
        SPI_PORT &= ~SPI_SCK; // SCK: \_
        SYNCDELAY;

        SPI_PORT |= SPI_SCK; // SCK: _/
        SYNCDELAY;

        if(SPI_PORT & SPI_MISO)
            *buf |= mask;
        SYNCDELAY;
        mask = mask >> 1;
    } while(mask != 0);
}


BYTE spi_read ( __xdata BYTE *buf, BYTE addr, BYTE length ) {
    BYTE bytes = 0;

    if ( length == 0 || length > 8 || run==0)
        return 0;

    SPI_PORT &= ~SPI_CS; SYNCDELAY; SYNCDELAY;


    spi_write_byte(addr | 0x80);
//    buf[0] = addr | 0x80; //debug
//    buf++; //debug

    SPI_PORT |= SPI_SD_T; SYNCDELAY; SYNCDELAY;

    for (; bytes<length; bytes++ ) {
        spi_read_byte(buf+bytes);
    }

spi_read_end:
    SPI_PORT |= SPI_CS; SYNCDELAY; SYNCDELAY;
    SPI_PORT &= ~SPI_SD_T; SYNCDELAY; SYNCDELAY;
    return bytes;
}

void spi_apply ()
{
    //write from i/o buf to actual register
    SPI_PORT |= SPI_IOUPDATE; SYNCDELAY; SYNCDELAY; SYNCDELAY; SYNCDELAY; //wait internal i/o update sysclk/4/4 = 16 cycles for 24mhz
    wait(5);
    SPI_PORT &= ~SPI_IOUPDATE;
    wait(5);
}

BYTE spi_write ( __xdata BYTE *buf, BYTE addr, BYTE length ) {
    BYTE bytes = 0;

    if ( length == 0 || length > 8 || run==0)
        return 0;

    SPI_PORT &= ~SPI_CS; SYNCDELAY; SYNCDELAY;

    spi_write_byte(addr);

    for (; bytes<length; bytes++ ) {
        spi_write_byte(buf[bytes]);
    }

spi_write_end:
    SPI_PORT |= SPI_CS; SYNCDELAY; SYNCDELAY;
    return bytes;
}

BYTE spi_read_ep0 () {
    BYTE i, b;
    b = ep0_payload_transfer;
    i = spi_read(EP0BUF, spi_addr, b);
    spi_addr++;
    return i;
}

//ADD_EP0_VENDOR_REQUEST((0x60,, 	// read from SPI
ADD_EP0_VENDOR_REQUEST(((0x60|0x80),, 	// read from SPI
//    eeprom_addr =  (SETUPDAT[3] << 8) | SETUPDAT[2];<-->// Address
    spi_addr =  SETUPDAT[2];	// Address
    EP0BCH = 0;
    EP0BCL = spi_read_ep0();
,,
    EP0BCH = 0;
    EP0BCL = spi_read_ep0();
));;

void spi_write_ep0 ( BYTE length ) {
    spi_write(EP0BUF, spi_addr, length);
    spi_addr++;
}

ADD_EP0_VENDOR_COMMAND((0x61,,	// write to SPI
    spi_write_bytes = 0;
//    eeprom_addr =  ( SETUPDAT[3] << 8) | SETUPDAT[2];<->// Address
    spi_addr =  SETUPDAT[2];	// Address
,,
    spi_write_ep0(EP0BCL);
    ));;

// apply_spi
ADD_EP0_VENDOR_COMMAND((0x62,,
	spi_apply();
,,
	NOP;
));;


// reset
ADD_EP0_VENDOR_COMMAND((0x81,,
	reset();
,,
	NOP;
));;

void reset () {
	OEA = bmBIT0 | bmBIT1 | bmBIT7;
	OEB = 0;
	OED = 0;
	MT_RESET = 1;
//	MT_MODE0 = 0;
//	MT_MODE1 = 0;
	
	EP2CS &= ~bmBIT0;			// clear stall bit
	EP6CS &= ~bmBIT0;			// clear stall bit

	IFCONFIG = bmBIT7 | bmBIT6 | bmBIT5 | 3;  // internal 48MHz clock, drive IFCLK output, slave FIFO interface
//	IFCONFIG = bmBIT7 | bmBIT5 | 3;  	  // internal 30MHz clock, drive IFCLK output, slave FIFO interface
	SYNCDELAY; 
                     
	REVCTL = 0x1;
	SYNCDELAY; 

	FIFORESET = 0x80;			// reset FIFO ...
	SYNCDELAY;
	FIFORESET = 2;				// ... for EP 2
	SYNCDELAY;
	FIFORESET = 0x00;
	SYNCDELAY;
	FIFORESET = 6;				// ... for EP 6
	SYNCDELAY;
	FIFORESET = 0x00;
	SYNCDELAY;

	EP2FIFOCFG = bmBIT0; 
	SYNCDELAY;
	EP2FIFOCFG = bmBIT3 | bmBIT0;           // EP2: AUTOIN, WORDWIDE
	SYNCDELAY;
	EP2AUTOINLENH = 2;                 	// 512 bytes 
	SYNCDELAY;
	EP2AUTOINLENL = 0;
	SYNCDELAY;

	EP6FIFOCFG = bmBIT0;         		
	SYNCDELAY;
	EP6FIFOCFG = bmBIT4 | bmBIT0;           // EP6: 0 -> 1 transition of AUTOOUT bit arms the FIFO, WORDWIDE
	SYNCDELAY;

	FIFOPINPOLAR = 0;
	SYNCDELAY; 
	PINFLAGSAB = 0xca;			// FLAGA: EP6: EF; FLAGB: EP2 FF
	SYNCDELAY; 

	wait(2);
	MT_RESET = 0;
        spi_init();
	run=1;
}


// include the main part of the firmware kit, define the descriptors, ...
#include[ztex.h]

void main(void)	
{
    init_USB();

    while (1) {	
    }
}


